package com.ieng.rms.services;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ieng.rms.entity.AlarmDetailConfig;
import com.ieng.rms.entity.AlarmDetails;
import com.ieng.rms.repository.AlarmDetailConfigRepository;
import com.ieng.rms.util.RedisUtil;
import com.ieng.rms.util.Util;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import static com.ieng.rms.util.Constants.*;
import static com.ieng.rms.util.Util.removeContractIdFromKey;

@Service
@Slf4j
public class AlarmDetailConfigService {

    private final AlarmDetailConfigRepository alarmDetailConfigRepository;
    private final RedisUtil redisUtil;
    private final AlarmDetailService alarmDetailService;

    public AlarmDetailConfigService(AlarmDetailConfigRepository alarmDetailConfigRepository, RedisUtil redisUtil, AlarmDetailService alarmDetailService) {
        this.alarmDetailConfigRepository = alarmDetailConfigRepository;
        this.redisUtil = redisUtil;
        this.alarmDetailService = alarmDetailService;
    }


    public List<AlarmDetailConfig> getAlarmDetailConfigList(Integer contractId) {
        return alarmDetailConfigRepository.getAlarmDetailConfigList(contractId);
    }

/*    public AlarmDetailConfig saveAlarmDetailConfig(AlarmDetailConfig alarmDetailConfig) {
        alarmDetailConfig.setChangeDate(LocalDateTime.now());
        alarmDetailConfigRepository.save(alarmDetailConfig);
        Constants.alarmDetailConfigList.clear();
        Constants.alarmDetailConfigList = alarmDetailConfigRepository.getAllAlarmDetailConfig();
*//*        Constants.alarmDetailConfigMap = new ConcurrentHashMap<>();
        Constants.alarmConfigMap = new ConcurrentHashMap<>();
        Constants.alarmDetailConfigList.forEach(p->{
            Constants.alarmDetailConfigMap.put(p.getAlarmName()+"_"+p.getContractId(),p.getCriticality());
            Constants.alarmConfigMap.put(p.getContractId()+"_"+p.getAlarmName(),p);
        });*//*
        return alarmDetailConfig;
    }*/

    public void deleteAlarmDetailConfig(Integer id) {
        AlarmDetailConfig model = getAlarmDetailConfig(id);
        model.setIsActive(false);
        alarmDetailConfigRepository.save(model);
    }

    public AlarmDetailConfig getAlarmDetailConfig(Integer id) {
        return alarmDetailConfigRepository
                .findById(id)
                .orElseThrow(() -> new NoSuchElementException("Alarm Detail Config Not Found By ID:" + id));
    }

/*    public AlarmDetailConfig getAlarmByPropertName() {
        return Constants.alarmDetailConfigList.stream()
                .filter(p -> p.getObjectProperty().equals("lvdDisconnect"))
                .findFirst().orElseThrow(() -> new RuntimeException("Alarm Config Property Not Found "));
    }*/

    public Map<String, Object> getAlarmConfigData() {
        try {
            // 🔹 Fetch from Redis first
            Map<String, String> alarmDetailConfigMap = redisUtil.getAlarmDetailConfigMap();
            Map<String, AlarmDetailConfig> alarmConfigMap = redisUtil.getAlarmConfigForNocMap();

            if (!alarmDetailConfigMap.isEmpty() && !alarmConfigMap.isEmpty()) {
                return Map.of(
                        ALARM_DETAIL_CONFIG_MAP, alarmDetailConfigMap,
                        ALARM_CONFIG_MAP_FOR_NOC, alarmConfigMap
                );
            }

            // 🔹 Fetch from DB if not in Redis
            List<AlarmDetailConfig> alarmList = alarmDetailConfigRepository.getAllAlarmDetailConfig();
            if (alarmList.isEmpty()) {
                return Map.of(
                        ALARM_DETAIL_CONFIG_MAP, new ConcurrentHashMap<>(),
                        ALARM_CONFIG_MAP_FOR_NOC, new ConcurrentHashMap<>()
                );
            }

            // Prepare maps
            Map<String, String> newAlarmDetailConfigMap = new ConcurrentHashMap<>();
            Map<String, AlarmDetailConfig> newAlarmConfigMap = new ConcurrentHashMap<>();

            alarmList.forEach(p -> {
                newAlarmDetailConfigMap.put(p.getAlarmName() + "_" + p.getContractId(), p.getCriticality());
                newAlarmConfigMap.put(p.getContractId() + "_" + p.getAlarmName(), p);
            });

            // 🔹 Save to Redis
            redisUtil.saveAlarmDetailConfigMap(newAlarmDetailConfigMap);
            redisUtil.saveAlarmConfigForNocMap(newAlarmConfigMap);

            // Return data
            return Map.of(
                    ALARM_DETAIL_CONFIG_MAP, newAlarmDetailConfigMap,
                    ALARM_CONFIG_MAP_FOR_NOC, newAlarmConfigMap
            );

        } catch (Exception e) {
            e.printStackTrace();
            return Map.of(
                    ALARM_DETAIL_CONFIG_MAP, new ConcurrentHashMap<>(),
                    ALARM_CONFIG_MAP_FOR_NOC, new ConcurrentHashMap<>()
            );
        }
    }


    public Map<String, String> getAlarmConfigMapByContractId(Integer contractId) {
        Map<String, String> alarmConfigMap = new HashMap<>();
        Map<String, Object> configData = getAlarmConfigData();
        ObjectMapper mapper = new ObjectMapper();

        Object rawData = configData.get(ALARM_DETAIL_CONFIG_MAP);

        Map<String, String> alarmDetailConfigMap = new HashMap<>();
        if (rawData instanceof Map) {
            try {
                alarmDetailConfigMap = mapper.convertValue(rawData, new TypeReference<>() {
                });
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        alarmDetailConfigMap.forEach((key, value) -> {
            if (key.contains(String.valueOf(contractId))) {
                alarmConfigMap.put(removeContractIdFromKey(key), value);
            }
        });
        return alarmConfigMap;
    }

    public Map<String, Map<String, AlarmDetails>> loadAlarmDetailsFromRedisOrDB(String siteId) {
        // Try to get all alarms from Redis
        Map<String, Map<String, AlarmDetails>> cachedAlarmDetailMap = redisUtil.getAlarmDetailsFromRedis();

        if (cachedAlarmDetailMap == null) {
            cachedAlarmDetailMap = new ConcurrentHashMap<>();
        }

        // Try to get cached alarms for the given siteId
        Map<String, AlarmDetails> siteAlarmDetails = cachedAlarmDetailMap.get(siteId);

        // ✅ Avoid duplicate fetches: Only query DB if data is NOT in Redis
        if (siteAlarmDetails == null || siteAlarmDetails.isEmpty()) {
            List<AlarmDetails> openAlarmsList = alarmDetailService.findOpenAlarms(siteId);

            if (!openAlarmsList.isEmpty()) { // Only update if there are alarms
                siteAlarmDetails = new ConcurrentHashMap<>();
                for (AlarmDetails alarm : openAlarmsList) {
                    String alarmKey = Util.addUnderscoreAgainstSpace(alarm.getAlarmName());
                    siteAlarmDetails.put(alarmKey, alarm);
                }

                // ✅ Update Redis only if new data is fetched
                cachedAlarmDetailMap.put(siteId, siteAlarmDetails);
                redisUtil.saveAlarmDetailsToRedis(cachedAlarmDetailMap);
            }
        }

        // ✅ Ensure we return the correct structure: { siteId: { alarmName: AlarmDetails } }
        Map<String, Map<String, AlarmDetails>> result = new HashMap<>();
        result.put(siteId, siteAlarmDetails != null ? siteAlarmDetails : new ConcurrentHashMap<>());

        return result;
    }


    public List<AlarmDetailConfig> loadAlarmDetailConfigFromRedisOrDB(Integer contractId) {
        // Try to get the list from Redis
        List<AlarmDetailConfig> cachedList = redisUtil.getAlarmDetailConfigFromRedis();

        if (cachedList == null || cachedList.isEmpty()) {
            // If not found in Redis, fetch from DB
            cachedList = alarmDetailConfigRepository.getAllAlarmDetailConfig();

            // Save to Redis
            redisUtil.saveAlarmDetailConfigToRedis(cachedList);
        }

        // Filter the list based on contractId
        return cachedList.stream()
                .filter(config -> config.getContractId().equals(contractId))
                .collect(Collectors.toList());
    }


    /**
     * Saves open alarms in Redis against a specific siteId.
     */
    public void saveAlarmsToRedis(String siteId, List<AlarmDetails> alarmsList) {
        if (siteId == null || alarmsList == null || alarmsList.isEmpty()) {
            return;
        }

        // Fetch existing alarms from Redis
        Map<String, Map<String, AlarmDetails>> cachedAlarmDetailMap = redisUtil.getAlarmDetailsFromRedis();
        if (cachedAlarmDetailMap == null) {
            cachedAlarmDetailMap = new ConcurrentHashMap<>();
        }

        // Get alarms for the site (if already in Redis)
        Map<String, AlarmDetails> siteAlarmDetails = cachedAlarmDetailMap.getOrDefault(siteId, new ConcurrentHashMap<>());

        for (AlarmDetails alarm : alarmsList) {
            String alarmKey = Util.addUnderscoreAgainstSpace(alarm.getAlarmName());

            // **Check if alarm already exists and is open**
            if (!siteAlarmDetails.containsKey(alarmKey) || siteAlarmDetails.get(alarmKey).getAlarmStatus() == 0) {
                siteAlarmDetails.put(alarmKey, alarm); // Add only if it's new or was previously closed
            }
        }

        // Update the Redis cache
        cachedAlarmDetailMap.put(siteId, siteAlarmDetails);
        redisUtil.saveAlarmDetailsToRedis(cachedAlarmDetailMap);
//        if (siteId == null || alarmsList == null || alarmsList.isEmpty()) {
//            return;
//        }
//
//        // Get existing alarms from Redis
//        Map<String, Map<String, AlarmDetails>> cachedAlarmDetailMap = redisUtil.getAlarmDetailsFromRedis();
//        if (cachedAlarmDetailMap == null) {
//            cachedAlarmDetailMap = new ConcurrentHashMap<>();
//        }
//
//        // Get or create site-specific alarm details
//        Map<String, AlarmDetails> siteAlarmDetails = cachedAlarmDetailMap.getOrDefault(siteId, new ConcurrentHashMap<>());
//
//        for (AlarmDetails alarm : alarmsList) {
//            String alarmKey = Util.addUnderscoreAgainstSpace(alarm.getAlarmName());
//
//            // **Check if the alarm already exists and is open**
//            if (siteAlarmDetails.containsKey(alarmKey)) {
//                AlarmDetails existingAlarm = siteAlarmDetails.get(alarmKey);
//
//                if (existingAlarm.getAlarmStatus() != null && existingAlarm.getAlarmStatus() == 1) { // Assuming '1' means open
//                    // **Skip adding this alarm because it's already open**
//                    continue;
//                }
//            }
//
//            // Add/Update the alarm in Redis
//            siteAlarmDetails.put(alarmKey, alarm);
//        }
//
//        // Save the updated alarm details back to Redis
//        cachedAlarmDetailMap.put(siteId, siteAlarmDetails);
//        redisUtil.saveAlarmDetailsToRedis(cachedAlarmDetailMap);
    }

    public List<AlarmDetailConfig> reloadAlarmDetailConfig() {
        try {
            // Remove the old data from Redis
            redisUtil.deleteKey(REDIS_ALARM_CONFIG_KEY);

            // Fetch fresh data from DB
            List<AlarmDetailConfig> freshConfigList = alarmDetailConfigRepository.getAllAlarmDetailConfig();

            // Save new data to Redis
            redisUtil.saveAlarmDetailConfigToRedis(freshConfigList);

            return freshConfigList;
        } catch (Exception e) {
            // Log error and return empty list if an exception occurs
            log.error("Error reloading alarm detail config from DB", e);
            return Collections.emptyList();
        }
    }
}


