package com.ieng.rms.services;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ieng.rms.dto.AlarmEscalationDTO;
import com.ieng.rms.dto.AlarmMaskingDTO;
import com.ieng.rms.dto.SiteWiseAlarmMaskingDTO;
import com.ieng.rms.entity.EscalationEmailEntity;
import com.ieng.rms.entity.SiteWiseAlarmMaskingConfig;
import com.ieng.rms.repository.EscalationEmailsRepository;
import com.ieng.rms.repository.SiteWiseAlarmConfigRepository;
import com.ieng.rms.util.RedisUtil;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class SiteWiseAlarmMaskingConfigService {

    private final SiteWiseAlarmConfigRepository siteWiseAlarmConfigRepository;

    private final EscalationEmailsRepository escalationEmailsRepository;

    private final RedisUtil redisUtil;

    public SiteWiseAlarmMaskingConfigService(SiteWiseAlarmConfigRepository repository, EscalationEmailsRepository escalationEmailsRepository, RedisUtil redisUtil) {
        this.siteWiseAlarmConfigRepository = repository;
        this.escalationEmailsRepository = escalationEmailsRepository;
        this.redisUtil = redisUtil;
    }

    public AlarmMaskingDTO getConfigsBySiteId(String siteId) {
        // Fetch all alarm configurations for the given siteId
        List<SiteWiseAlarmMaskingConfig> configs = siteWiseAlarmConfigRepository.findAllBySiteId(siteId);

        if (configs.isEmpty()) {
            return null; // Return null or handle no records found scenario
        }

        // Extract contractId (assuming all alarms belong to the same contract)
        Integer contractId = configs.get(0).getContractId();

        // Convert alarm configurations to DTOs
        List<SiteWiseAlarmMaskingDTO> alarmDTOs = configs.stream().map(alarm -> {
            List<String> translations = parseJsonTranslations(alarm.getTranslation());
            List<AlarmEscalationDTO> escalations = getEscalationsForAlarm(alarm);
            return new SiteWiseAlarmMaskingDTO(alarm.getId(), alarm.getAlarmName(), alarm.getAlarmMaskingName(),
                    alarm.getCriticality(), translations, alarm.getEscalationEnable(), alarm.getAlarmEnabled(),
                    alarm.getIsNocAlarm(), escalations);
        }).collect(Collectors.toList());

        // Return a single AlarmMaskingDTO for the given siteId
        return new AlarmMaskingDTO(alarmDTOs, siteId, contractId);
    }


    public ResponseEntity<?> saveConfig(AlarmMaskingDTO request) {
        ObjectMapper objectMapper = new ObjectMapper();
        List<SiteWiseAlarmMaskingDTO> savedAlarmConfigs = new ArrayList<>();
        List<String> errorMessages = new ArrayList<>();

        // Validate Site ID and Contract ID
        if (request.getSiteId() == null || request.getSiteId().trim().isEmpty()) {
            errorMessages.add("Site ID is required and cannot be empty.");
        }
        if (request.getContractId() == null || request.getContractId().toString().trim().isEmpty()) {
            errorMessages.add("Contract ID is required and cannot be empty.");
        }
        if (request.getAlarmConfig() == null || request.getAlarmConfig().isEmpty()) {
            errorMessages.add("Alarms Masking are required and cannot be empty.");
        }

        // Validate alarms and check for duplicates in request
        Set<String> alarmNames = new HashSet<>();
        for (SiteWiseAlarmMaskingDTO config : request.getAlarmConfig()) {
            if (config.getAlarmName() == null || config.getAlarmName().trim().isEmpty()) {
                errorMessages.add("Alarm Name is required and cannot be empty.");
            }
            if (config.getAlarmMaskingName() == null || config.getAlarmMaskingName().trim().isEmpty()) {
                errorMessages.add("Alarm Masking Name is required and cannot be empty.");
            }
            if (!alarmNames.add(config.getAlarmName())) {
                errorMessages.add("Duplicate Alarm Name '" + config.getAlarmName() + "' found in request.");
            }

            // Check if alarm name already exists in DB
            boolean exists = siteWiseAlarmConfigRepository.findOneBySiteIdAndAlarmName(request.getSiteId(), config.getAlarmName()).isPresent();
            if (exists) {
                errorMessages.add("Alarm Name '" + config.getAlarmName() + "' already exists for this Site ID.");
            }
        }

        // If there are errors, return them
        if (!errorMessages.isEmpty()) {
            return ResponseEntity.badRequest().body(Map.of("errors", errorMessages));
        }

        // Save valid alarms
        for (SiteWiseAlarmMaskingDTO config : request.getAlarmConfig()) {
            SiteWiseAlarmMaskingConfig siteWiseAlarmMaskingConfig = new SiteWiseAlarmMaskingConfig();
            siteWiseAlarmMaskingConfig.setAlarmName(config.getAlarmName());
            siteWiseAlarmMaskingConfig.setAlarmMaskingName(config.getAlarmMaskingName());
            siteWiseAlarmMaskingConfig.setCriticality(config.getCriticality());
            siteWiseAlarmMaskingConfig.setContractId(request.getContractId());
            siteWiseAlarmMaskingConfig.setSiteId(request.getSiteId());
            siteWiseAlarmMaskingConfig.setIsNocAlarm(config.getIsNocAlarm());
            siteWiseAlarmMaskingConfig.setEscalationEnable(config.getEscalationEnable());
            siteWiseAlarmMaskingConfig.setAlarmEnabled(config.getAlarmEnabled());

            // Convert List<String> translation to JSON String
            try {
                String jsonTranslation = objectMapper.writeValueAsString(config.getTranslation());
                siteWiseAlarmMaskingConfig.setTranslation(jsonTranslation);
            } catch (JsonProcessingException e) {
                return ResponseEntity.internalServerError().body(Map.of("error", "Failed to convert translation to JSON"));
            }

            // Handle Escalations
            List<EscalationEmailEntity> escalationEntities = new ArrayList<>();
            if (config.getEscalations() != null) {
                for (AlarmEscalationDTO escalationDTO : config.getEscalations()) {
                    EscalationEmailEntity escalationEntity = escalationEmailsRepository.findFirstByUserIdAndEmail(escalationDTO.getUserId(), escalationDTO.getEmail()).orElseGet(() -> {
                        EscalationEmailEntity newEscalation = new EscalationEmailEntity();
                        newEscalation.setUserId(escalationDTO.getUserId());
                        newEscalation.setEmail(escalationDTO.getEmail());
                        return escalationEmailsRepository.save(newEscalation);
                    });

                    escalationEntities.add(escalationEntity);
                }
            }
            siteWiseAlarmMaskingConfig.setEscalations(escalationEntities);

            // Save the alarm configuration
            SiteWiseAlarmMaskingConfig maskingConfig = siteWiseAlarmConfigRepository.save(siteWiseAlarmMaskingConfig);
            config.setId(maskingConfig.getId());
            savedAlarmConfigs.add(config);
        }

        // Return success response
        return ResponseEntity.ok(new AlarmMaskingDTO(savedAlarmConfigs, request.getSiteId(), request.getContractId()));
    }

    public ResponseEntity<?> deleteAlarms(List<Long> alarmIds) {
        List<String> errorMessages = new ArrayList<>();

        // Validate input
        if (alarmIds == null || alarmIds.isEmpty()) {
            errorMessages.add("Alarm Ids are required and cannot be empty.");
            return ResponseEntity.badRequest().body(Map.of("errors", errorMessages));
        }

        // Retrieve alarms that actually exist
        List<SiteWiseAlarmMaskingConfig> alarmsToDelete = siteWiseAlarmConfigRepository.findAllById(alarmIds);

        // Extract the IDs of the existing alarms
        List<Long> existingIds = alarmsToDelete.stream()
                .map(SiteWiseAlarmMaskingConfig::getId)
                .collect(Collectors.toList());

        // Identify IDs that do not exist
        List<Long> nonExistentIds = alarmIds.stream()
                .filter(id -> !existingIds.contains(id))
                .collect(Collectors.toList());

     /*   if (!nonExistentIds.isEmpty()) {
            errorMessages.add("The following alarm IDs do not exist: " + nonExistentIds);
            return ResponseEntity.badRequest().body(Map.of("errors", errorMessages));
        }*/

        // Proceed to delete the alarms
        siteWiseAlarmConfigRepository.deleteAll(alarmsToDelete);

        return ResponseEntity.ok(Map.of("message", "Alarm deleted successfully."));
    }


    public List<AlarmMaskingDTO> getAllAlarmMaskingData() {
        return getAlarmMaskingData(siteWiseAlarmConfigRepository.findAll());
    }

    public List<AlarmMaskingDTO> getAllAlarmMaskingEscalationEnable() {
        return getAlarmMaskingData(siteWiseAlarmConfigRepository.findAllByEscalationEnableIsTrue());
    }

    private List<AlarmMaskingDTO> getAlarmMaskingData(List<SiteWiseAlarmMaskingConfig> alarms) {
        // Group alarms by contractId, then by siteId
        Map<Integer, Map<String, List<SiteWiseAlarmMaskingConfig>>> groupedAlarms = alarms.stream().collect(Collectors.groupingBy(SiteWiseAlarmMaskingConfig::getContractId, Collectors.groupingBy(SiteWiseAlarmMaskingConfig::getSiteId)));

        List<AlarmMaskingDTO> result = new ArrayList<>();

        for (Map.Entry<Integer, Map<String, List<SiteWiseAlarmMaskingConfig>>> contractEntry : groupedAlarms.entrySet()) {
            Integer contractId = contractEntry.getKey();
            Map<String, List<SiteWiseAlarmMaskingConfig>> siteMap = contractEntry.getValue();

            for (Map.Entry<String, List<SiteWiseAlarmMaskingConfig>> siteEntry : siteMap.entrySet()) {
                String siteId = siteEntry.getKey();
                List<SiteWiseAlarmMaskingConfig> siteAlarms = siteEntry.getValue();

                // Convert alarms to DTO format
                List<SiteWiseAlarmMaskingDTO> alarmDTOs = siteAlarms.stream().map(alarm -> {
                    List<String> translations = parseJsonTranslations(alarm.getTranslation());
                    List<AlarmEscalationDTO> escalations = getEscalationsForAlarm(alarm);
                    return new SiteWiseAlarmMaskingDTO(alarm.getId(), alarm.getAlarmName(), alarm.getAlarmMaskingName(), alarm.getCriticality(), translations, alarm.getEscalationEnable(), alarm.getAlarmEnabled(), alarm.getIsNocAlarm(), escalations);
                }).collect(Collectors.toList());

                // Create DTO object
                result.add(new AlarmMaskingDTO(alarmDTOs, siteId, contractId));
            }
        }
        return result;
    }

    private List<String> parseJsonTranslations(String json) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            return objectMapper.readValue(json, new TypeReference<List<String>>() {
            });
        } catch (Exception e) {
            return Collections.emptyList();
        }
    }

    private List<AlarmEscalationDTO> getEscalationsForAlarm(SiteWiseAlarmMaskingConfig alarm) {
        return alarm.getEscalations().stream().map(escalation -> new AlarmEscalationDTO(escalation.getUserId(), escalation.getEmail())).collect(Collectors.toList());
    }

    @Transactional
    public ResponseEntity<?> updateConfig(AlarmMaskingDTO request) {
        List<SiteWiseAlarmMaskingDTO> updatedAlarmConfigs = new ArrayList<>();
        ObjectMapper objectMapper = new ObjectMapper();

        // Validate request parameters
        if (request.getSiteId() == null || request.getSiteId().trim().isEmpty()) {
            return ResponseEntity.badRequest().body(Map.of("message", "Site ID is required and cannot be empty."));
        }
        if (request.getContractId() == null || request.getContractId().toString().trim().isEmpty()) {
            return ResponseEntity.badRequest().body(Map.of("message", "Contract ID is required and cannot be empty."));
        }
        if (request.getAlarmConfig() == null || request.getAlarmConfig().isEmpty()) {
            return ResponseEntity.badRequest().body(Map.of("message", "Alarms Masking are required and cannot be empty."));
        }
        for (SiteWiseAlarmMaskingDTO config : request.getAlarmConfig()) {

            if (config.getAlarmName() == null || config.getAlarmName().trim().isEmpty()) {
                return ResponseEntity.badRequest().body(Map.of("message", "Alarm Name is required and cannot be empty."));
            }
            if (config.getAlarmMaskingName() == null || config.getAlarmMaskingName().trim().isEmpty()) {
                return ResponseEntity.badRequest().body(Map.of("message", "Alarm Masking Name is required and cannot be empty."));
            }


            // Fetch existing record by ID (since alarm name can be updated)
            SiteWiseAlarmMaskingConfig existingConfig = null;
            if (config.getId() != null) {
                existingConfig = siteWiseAlarmConfigRepository.findById(config.getId()).orElse(null);
            }

            // If the config doesn't exist, create a new one
            if (existingConfig == null) {
                existingConfig = new SiteWiseAlarmMaskingConfig();
                existingConfig.setSiteId(request.getSiteId());
                existingConfig.setContractId(request.getContractId());
            }

            // Update fields (including new alarm name if changed)
            existingConfig.setAlarmName(config.getAlarmName());
            existingConfig.setAlarmMaskingName(config.getAlarmMaskingName());
            existingConfig.setCriticality(config.getCriticality());
            existingConfig.setIsNocAlarm(config.getIsNocAlarm());
            existingConfig.setEscalationEnable(config.getEscalationEnable());
            existingConfig.setAlarmEnabled(config.getAlarmEnabled());

            // Convert List<String> translation to JSON String
            try {
                String jsonTranslation = objectMapper.writeValueAsString(config.getTranslation());
                existingConfig.setTranslation(jsonTranslation);
            } catch (JsonProcessingException e) {
                return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(Map.of("message", "Failed to convert translation to JSON."));
            }

            // Handle Escalations
            List<EscalationEmailEntity> escalationEntities = new ArrayList<>();
            if (config.getEscalations() != null) {
                for (AlarmEscalationDTO escalationDTO : config.getEscalations()) {
                    EscalationEmailEntity escalationEntity = escalationEmailsRepository.findFirstByUserIdAndEmail(escalationDTO.getUserId(), escalationDTO.getEmail()).orElseGet(() -> {
                        EscalationEmailEntity newEscalation = new EscalationEmailEntity();
                        newEscalation.setUserId(escalationDTO.getUserId());
                        newEscalation.setEmail(escalationDTO.getEmail());
                        return escalationEmailsRepository.save(newEscalation);
                    });

                    escalationEntities.add(escalationEntity);
                }
            }
            existingConfig.setEscalations(escalationEntities);

            // Save the updated or newly created alarm configuration
            SiteWiseAlarmMaskingConfig maskingConfig = siteWiseAlarmConfigRepository.save(existingConfig);
            config.setId(maskingConfig.getId());
            updatedAlarmConfigs.add(config);
            redisUtil.deleteAlarmMaskingConfig(request.getSiteId());

        }

        // Return success response
        return ResponseEntity.ok(new AlarmMaskingDTO(updatedAlarmConfigs, request.getSiteId(), request.getContractId()));
    }

    /**
     * Fetches alarm configurations from Redis cache; if not available, retrieves from DB and caches it.
     */
    public Map<String, SiteWiseAlarmMaskingConfig> getAlarmMaskingConfigMap(String siteId) {
        // Attempt to get data from Redis
        Map<String, SiteWiseAlarmMaskingConfig> cachedData = redisUtil.getAlarmMaskingConfigFromRedis(siteId);

        if (cachedData != null && !cachedData.isEmpty()) {
            return cachedData; // Return cached data if found
        }

        // Fetch from DB if not in Redis
        List<SiteWiseAlarmMaskingConfig> configList = siteWiseAlarmConfigRepository.findAllBySiteId(siteId);
        if (configList == null) {
            configList = Collections.emptyList();
        }

        Map<String, SiteWiseAlarmMaskingConfig> alarmConfigMap = configList.stream()
                .collect(Collectors.toMap(
                        SiteWiseAlarmMaskingConfig::getAlarmName,
                        config -> config,
                        (existing, replacement) -> replacement
                ));

        // Check if the map is not null and not empty before saving to Redis
        if (!alarmConfigMap.isEmpty()) {
            try {
                redisUtil.saveAlarmMaskingConfigToRedis(siteId, alarmConfigMap);
            } catch (Exception e) {
                // Log the exception if needed
                e.printStackTrace();
            }
        }

        return alarmConfigMap;
    }

}
