package com.ieng.rms.subscriber;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.ieng.rms.dto.ContractDTO;
import com.ieng.rms.entity.AlarmDetailConfig;
import com.ieng.rms.entity.AlarmDetails;
import com.ieng.rms.entity.EscalationEmailEntity;
import com.ieng.rms.entity.SiteWiseAlarmMaskingConfig;
import com.ieng.rms.exception.InternalServerException;
import com.ieng.rms.models.PacketModel;
import com.ieng.rms.models.SiteDetailsDTO;
import com.ieng.rms.repository.AlarmDetailRepository;
import com.ieng.rms.services.AlarmDetailConfigService;
import com.ieng.rms.services.RmsCoreClientService;
import com.ieng.rms.services.SiteWiseAlarmMaskingConfigService;
import com.ieng.rms.util.DateUtil;
import com.ieng.rms.util.RedisUtil;
import com.ieng.rms.util.Util;
import com.ieng.rms.websocket.AlarmWebSocketHandler;
import lombok.SneakyThrows;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.Period;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import static com.ieng.rms.util.Constants.contractMap;


@Component
public class AlarmDetailSubscriber {

    @Value("${spring.mail.alert.username}")
    private String alertFromAddress;

    private static final Logger logger = LogManager.getLogger(AlarmDetailSubscriber.class);

    private final AlarmDetailRepository alarmRepository;

    private final RedisUtil redisUtil;

    private final RmsCoreClientService rmsCoreClientService;

    private final AlarmWebSocketHandler alarmWebSocketHandler;

    private final SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService;

    private final JavaMailSender alertJavaMailSender;

    private final AlarmDetailConfigService alarmDetailConfigService;


    public AlarmDetailSubscriber(AlarmDetailRepository alarmRepository, RedisUtil redisUtil, RmsCoreClientService rmsCoreClientService, AlarmWebSocketHandler alarmWebSocketHandler, SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService, JavaMailSender alertJavaMailSender, AlarmDetailConfigService alarmDetailConfigService) {
        this.alarmRepository = alarmRepository;
        this.redisUtil = redisUtil;
        this.rmsCoreClientService = rmsCoreClientService;
        this.alarmWebSocketHandler = alarmWebSocketHandler;
        this.siteWiseAlarmMaskingConfigService = siteWiseAlarmMaskingConfigService;
        this.alertJavaMailSender = alertJavaMailSender;
        this.alarmDetailConfigService = alarmDetailConfigService;
    }

    //    @JmsListener(destination = "${jsa.activemq.sonic-alarms}", containerFactory = "jmsListenerContainerQueue")
    //	@Async(value = "alarmDetailThreadExecutor")
    public void receive(PacketModel model) {
        try {
            SiteDetailsDTO site = rmsCoreClientService.getSiteDetailsBySiteId(model.getSiteID());

            //redis key(clientId) value(dateTime)
            //compare model.dateTime with redis last process datetime of packet.
            //if model.dateTime is greater than redis last process datetime of packet or redis is empty let's proceed the process.
            //otherwise return
            String inMemoryPacketTime = redisUtil.getHistoryPacketByKey(redisUtil.generateKeyForPacketHistory(model.getClientId() + "-" + model.getSiteID()));
            if (inMemoryPacketTime == null || DateUtil.getLocalDateTime(model.getDateAndTimeStamp()).isAfter(DateUtil.getStringToLocalDateTime(inMemoryPacketTime))) {

                Map<String, Object> packetModelMap = new ObjectMapper().convertValue(model, Map.class);
                Map<String, Map<String, AlarmDetails>> alarmDetails = alarmDetailConfigService.loadAlarmDetailsFromRedisOrDB(site.getSiteId());
                List<AlarmDetails> alarmsList = new ArrayList<>();
                additionStaticAlarmsInPacketMap(model, packetModelMap);
                Map<String, Map<String, AlarmDetails>> alarmsMap = setAlarmOpen(model, model.getPacketId(), site, packetModelMap, alarmDetails);
                if (alarmsMap.containsKey(model.getSiteID())) {
                    for (Map.Entry<String, AlarmDetails> entry : alarmsMap.get(model.getSiteID()).entrySet()) {
                        alarmsList.add(entry.getValue());
                    }
                    alarmsList = alarmsList.stream().filter(p -> p.getId() == null).collect(Collectors.toList());
                    saveOpenAlarms(model, alarmsList, site);
                }
                List<AlarmDetails> closedList = setAlarmClose(model, model.getPacketId(), packetModelMap, alarmDetails, site.getContractParentId());
                if (!closedList.isEmpty()) {
                    alarmRepository.saveAll(closedList);
//                    saveClosedAlarms(site.getSiteId(), closedList);
                    alarmsEscalationConfig(model, closedList, false, site);
                }
                redisUtil.addToRedisForPacketHistory(redisUtil.generateKeyForPacketHistory(model.getClientId() + "-" + model.getSiteID()), DateUtil.convertDateToString(model.getDateAndTimeStamp()));
            } else {
                logger.info("Packet not added in redis ");
            }
            //put model.datetime in memory map
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("Exception occur while throwing Stream to UI" + e);
        }
    }

    private void saveOpenAlarms(PacketModel packetModel, List<AlarmDetails> alarmsList, SiteDetailsDTO siteDetailsDTO) {
        // Return early if there are no alarms to process
        if (alarmsList.isEmpty()) {
            return;
        }
        alarmsEscalationConfig(packetModel, alarmsList, true, siteDetailsDTO);
        alarmRepository.saveAll(alarmsList);
        alarmDetailConfigService.saveAlarmsToRedis(packetModel.getSiteID(), alarmsList);


    }


    private void alarmsEscalationConfig(PacketModel packetModel, List<AlarmDetails> alarmsList, boolean isOpen, SiteDetailsDTO siteDetailsDTO) {
        String siteId = packetModel.getSiteID();

        // Fetch site-wise alarm configurations from cache or DB
        Map<String, SiteWiseAlarmMaskingConfig> alarmConfigMap = siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(siteId);

        if (alarmConfigMap.isEmpty()) {
            return; // No escalation needed
        }

        // Extract escalated alarm names where both alarm and escalation are enabled
        Set<String> escalatedAlarmNames = alarmConfigMap.values().stream()
                .filter(config -> config.getEscalationEnable() && config.getAlarmEnabled())
                .map(SiteWiseAlarmMaskingConfig::getAlarmName)
                .collect(Collectors.toSet());

        // Identify alarms that require escalation and update their details
        List<AlarmDetails> alarmsToEscalate = alarmsList.stream()
                .filter(alarm -> escalatedAlarmNames.contains(Util.addUnderscoreAgainstSpace(alarm.getAlarmName()))) // Normalize alarm name
                .map(alarm -> {
                    try {
                        // Clone the existing alarm to avoid modifying the original object
                        AlarmDetails updatedAlarm = (AlarmDetails) alarm.clone();

                        // Get the masking name or use the original name if not found
                        String normalizedAlarmName = alarm.getAlarmName();
                        String maskingName = alarmConfigMap.containsKey(Util.addUnderscoreAgainstSpace(normalizedAlarmName))
                                ? alarmConfigMap.get(Util.addUnderscoreAgainstSpace(normalizedAlarmName)).getAlarmMaskingName()
                                : alarm.getAlarmName();

                        updatedAlarm.setAlarmMaskingName(maskingName); // Replace alarm name with masking name
                        return updatedAlarm;
                    } catch (CloneNotSupportedException e) {
                        throw new RuntimeException("Error cloning AlarmDetails object", e);
                    }
                })
                .toList();

        if (alarmsToEscalate.isEmpty()) {
            return; // No alarms to escalate
        }

        // Map emails to the alarms they should receive
        Map<String, Set<AlarmDetails>> emailToAlarmsMap = new HashMap<>();

        for (AlarmDetails alarm : alarmsToEscalate) {
            String alarmKey = Util.addUnderscoreAgainstSpace(alarm.getAlarmName());
            if (!alarmConfigMap.containsKey(alarmKey)) continue;

            SiteWiseAlarmMaskingConfig config = alarmConfigMap.get(alarmKey);
            if (Boolean.TRUE.equals(!config.getEscalationEnable()) || config.getEscalations() == null) continue;

            for (EscalationEmailEntity escalation : config.getEscalations()) {
                emailToAlarmsMap.computeIfAbsent(escalation.getEmail(), k -> new HashSet<>()).add(alarm);
            }
        }

        // ✅ Send only one email per recipient containing all their alarms
        emailToAlarmsMap.forEach((email, alarms) ->
                escalationEmailTemplate(siteDetailsDTO, new ArrayList<>(alarms), List.of(email), isOpen)
        );
    }


    private void escalationEmailTemplate(SiteDetailsDTO siteDetailsDTO, List<AlarmDetails> alarms, List<String> emailRecipients, boolean isOpen) {
        if (emailRecipients == null || emailRecipients.isEmpty()) {
            return; // No emails to send escalation
        }
        // Prepare email content
        String emailSubject = "RMS Alarm Notification ||" + getContractById(siteDetailsDTO.getContractParentId()).getName() + "||" + siteDetailsDTO.getSiteId();
        String emailBody = createEscalationEmailTemplate(siteDetailsDTO.getSiteId(), alarms, isOpen);
        // Convert List<String> to String[]
        String[] emailRecipientsArray = emailRecipients.toArray(new String[0]);
        // Send email
        sendEscalationAlertEmail(emailRecipientsArray, emailSubject, emailBody);
    }

    private String createEscalationEmailTemplate(String siteId, List<AlarmDetails> alarms, boolean isOpen) {
        // Convert the list of alarms to a table row format
        StringBuilder alarmRows = new StringBuilder();

        for (AlarmDetails alarm : alarms) {
            String severityClass = "";

            switch (alarm.getAlarmCriticality()) {
                case "Critical":
                    severityClass = "critical";
                    break;
                case "Major":
                    severityClass = "major";
                    break;
                case "Minor":
                    severityClass = "minor";
                    break;
            }
            alarmRows.append("<tr>")
                    .append("<td>").append(alarm.getAlarmMaskingName()).append("</td>")
                    .append("<td>").append(alarm.getFromTime()).append("</td>").append("<td class='").append(severityClass).append("'>").append(alarm.getAlarmCriticality()).append("</td>")
                    .append("<td>").append(alarm.getAlarmStatus() == 1 ? "Open" : "Closed").append("</td>");

            // Add Elapsed Time only if isOpen is false
            if (!isOpen) {
                alarmRows.append("<td>").append(alarm.getElapsedTime()).append("</td>");
            }

            alarmRows.append("</tr>");
        }


        return "<html>"
                + "<head>"
                + "<style>"
                + "body { font-family: Arial, sans-serif; color: #333; background-color: #f4f4f4; padding: 20px; }"
                + ".container { max-width: 600px; background: #ffffff; padding: 20px; border-radius: 10px; box-shadow: 0px 4px 8px rgba(0, 0, 0, 0.1); }"
                + "h3 { color: #2e7d32; text-align: center; }"
                + "p { font-size: 14px; color: #555; }"
                + "table { width: 100%; border-collapse: collapse; margin-top: 15px; }"
                + "th, td { border: 1px solid #ddd; padding: 10px; text-align: left; }"
                + "th { background-color: #000000; color: #ffffff; }"
                + "tr:nth-child(even) { background-color: #f9f9f9; }"
                + "tr:nth-child(odd) { background-color: #ffffff; }"
                + ".critical { background-color: red; color: white; }"
                + ".major { background-color: yellow; color: black; }"
                + ".minor { background-color: green; color: white; }"
                + ".footer { margin-top: 30px; font-size: 14px; color: #777; border-top: 1px solid #ddd; padding-top: 10px; text-align: left; }"
                + ".signature { font-size: 14px; font-weight: bold; color: #333; }"
                + "</style>"
                + "</head>"
                + "<body>"
                + "<div class='container'>"
                + "<p>"
                + (isOpen
                ? "The following alarms are currently open for site <strong>" + siteId + "</strong>:"
                : "The following alarms are currently closed for site <strong>" + siteId + "</strong>:")
                + "</p>"
                + "<table>"
                + "<tr>"
                + "<th>Alarm Name</th>"
                + "<th>Alarm Time</th>"
                + "<th>Severity</th>"
                + "<th>Status</th>"
                + (isOpen ? "" : "<th>Elapsed Time</th>")
                + "</tr>"
                + alarmRows // This should include logic to apply the right CSS class dynamically
                + "</table>"
                + "<p>Please take necessary action.</p>"
                + "</div>"
                + "<div class='footer'>"
                + "<p class='signature'>Best Regards,<br>RMS</p>"
                + "</div>"
                + "</body>"
                + "</html>";

    }


    private List<AlarmDetails> setAlarmClose(PacketModel model, Integer packetId, Map<String, Object> packetModelMap, Map<String, Map<String, AlarmDetails>> alarmDetails, Integer contractId) {
        List<AlarmDetails> openAlarms;
        List<AlarmDetails> finalList = new ArrayList<>();
        if (alarmDetails.containsKey(model.getSiteID())) {
            openAlarms = alarmDetails.get(model.getSiteID())
                    .values().stream()
                    .toList();

            for (AlarmDetails aModel : openAlarms) {
                String alarmSource = Util.addUnderscoreAgainstSpace(aModel.getAlarmName());
                Integer alarmStatus = Util.getAlarmsStatusOf(getAlarmDetailConfigModel(aModel, contractId), model, packetModelMap);

                if (!Objects.equals(aModel.getAlarmStatus(), alarmStatus) && DateUtil.differenceInMilli(aModel.getFromTime(), DateUtil.getLocalDateTime(model.getDateAndTimeStamp())) > 0) {
                    aModel.setFinalPacketId(String.valueOf(packetId));
                    aModel.setToTime(DateUtil.getLocalDateTime(model.getDateAndTimeStamp()));
                    aModel.setElapsedTime(calculateElapsedTime(aModel));
                    aModel.setAlarmStatus(alarmStatus);
                    if (siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(aModel.getSiteId()) != null && siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(aModel.getSiteId()).containsKey(Util.addUnderscoreAgainstSpace(aModel.getAlarmName()))) {
                        aModel.setAlarmMaskingName(siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(aModel.getSiteId()).get(Util.addUnderscoreAgainstSpace(aModel.getAlarmName())).getAlarmMaskingName());
                    } else {
                        aModel.setAlarmMaskingName("");
                    }
                    Period period = Period.between(aModel.getFromTime().toLocalDate(), aModel.getToTime().toLocalDate());

                    if (period.getDays() > 0) {
                        checkAlarmCloseInstance(aModel, finalList);
                    } else {
                        finalList.add(aModel);
                    }

                    if (alarmDetails.containsKey(model.getSiteID())) {
                        alarmDetails.get(model.getSiteID()).remove(alarmSource);
                    }

                    // **REMOVE FROM REDIS**
                    redisUtil.removeAlarmDetailFromRedis(model.getSiteID(), alarmSource);
                }
            }

        }
        return finalList;
    }

    private String calculateElapsedTime(AlarmDetails aModel) {
        return DateUtil.millisecondsToTime(DateUtil.differenceInMilli(aModel.getFromTime(), aModel.getToTime()));
    }

    private void checkAlarmCloseInstance(AlarmDetails aModel, List<AlarmDetails> finalList) {

        try {
            List<LocalDate> allDates = DateUtil.getAllDates(aModel.getFromTime(), aModel.getToTime());
            if (!allDates.isEmpty()) {
                fillAlarmInstanceForWholeDay(allDates, aModel, finalList);
            } else {

                aModel.setFromTime(DateUtil.atStartOfDay(aModel.getToTime().toLocalDate()));
                aModel.setElapsedTime(calculateElapsedTime(aModel));
                aModel.setId(null);
                finalList.add(aModel);
            }

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void fillAlarmInstanceForWholeDay(List<LocalDate> allDates, AlarmDetails aModel, List<AlarmDetails> finalList) {
        LocalDate startDate = aModel.getFromTime().toLocalDate();

        allDates.forEach(p -> {
            AlarmDetails alarmModel = null;
            try {
                alarmModel = (AlarmDetails) aModel.clone();
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (p.isEqual(startDate)) {
                alarmModel.setFromTime(aModel.getFromTime());
            } else {
                alarmModel.setFromTime(p.atStartOfDay());
                alarmModel.setId(null);
            }

            if (alarmModel.getToTime().toLocalDate().isEqual(p)) {
                alarmModel.setToTime(alarmModel.getToTime());
            } else {
                alarmModel.setToTime(DateUtil.atEndOfDay(p));
            }
            alarmModel.setElapsedTime(calculateElapsedTime(alarmModel));

            finalList.add(alarmModel);
        });
    }

    @SneakyThrows
    private AlarmDetailConfig getAlarmDetailConfigModel(AlarmDetails aModel, Integer contractId) {
        List<AlarmDetailConfig> alarmConfigList;

        AlarmDetailConfig alarmDetailConfig = null;
        alarmConfigList = alarmDetailConfigService.loadAlarmDetailConfigFromRedisOrDB(contractId);
        try {
            alarmDetailConfig = alarmConfigList.stream().filter(p -> p.getAlarmName().equals(Util.addUnderscoreAgainstSpace(aModel.getAlarmName()))).findFirst().get();
        } catch (Exception e) {
            logger.info("Alarm Configuration Does not exists: " + aModel.getAlarmName() + ",  site: " + aModel.getSiteId());
        }
        return alarmDetailConfig;
    }


    private Map<String, Map<String, AlarmDetails>> setAlarmOpen(PacketModel model, Integer packetId, SiteDetailsDTO site, Map<String, Object> packetModelMap, Map<String, Map<String, AlarmDetails>> alarmDetails) {
        AlarmDetails view;
        List<AlarmDetailConfig> alarmConfigList;
        alarmConfigList = alarmDetailConfigService.loadAlarmDetailConfigFromRedisOrDB(site.getContractParentId());
        for (AlarmDetailConfig alarmConfig : alarmConfigList) {
            view = new AlarmDetails();
            view.setAlarmName(Util.removeUnderscoreWithSpace(alarmConfig.getAlarmName()));
            view.setAlarmStatus(Util.getAlarmsStatusOf(alarmConfig, model, packetModelMap));
            view.setSiteId(model.getSiteID());
            view.setInitialPacketId(String.valueOf(packetId));
            view.setFromTime(DateUtil.getLocalDateTime(model.getDateAndTimeStamp()));
            view.setAlarmCriticality(alarmConfig.getCriticality());
            view.setClientId(model.getClientId());
            if (!alarmDetails.containsKey(model.getSiteID())) {
                alarmDetails.put(model.getSiteID(), new ConcurrentHashMap<>());
            } else {
                try {
                    if (1 == view.getAlarmStatus() || view.getAlarmStatus() > 0) {
                        if (!alarmDetails.get(model.getSiteID()).containsKey(Util.addUnderscoreAgainstSpace(view.getAlarmName()))) {
                            alarmDetails.get(model.getSiteID()).put(Util.addUnderscoreAgainstSpace(view.getAlarmName()), view);
                        }
                    }
                } catch (Exception ex) {
                    logger.debug("Site Name:: " + view.getSiteId() + ", Alarm Detail name:" + view.getAlarmName() + ", Status:: " + view.getAlarmStatus());
                    logger.debug("Message:: ", ex);
                }
            }
        }

        return alarmDetails;
    }

    /*
     * Addition Alarms Key, Values in latestPacketMap for alarmReport and Monitor
     * which are not exists in Excel and packet as these alarms are generated by multiple parameters.
     *
     * Object Property Should be same as defined here in key for clarity purpose
     * */
    private void additionStaticAlarmsInPacketMap(PacketModel model, Map<String, Object> packetModelMap) {
        packetModelMap.put("Battery_Fault", checkBatteryFault(model));
        packetModelMap.put("PV_Module_Fail", checkPvModuleFail(model));
        packetModelMap.put("Li_Comm_Fault", checkLiCommFault(model));
        //packetModelMap.put("DC_HV_module_Disconnect", checkDcHvModuleDisconnect(model));
        //packetModelMap.put("High_Temp_Module_Shutdown", checkHighTemperatureModuleShutdown(model));
        packetModelMap.put("DC_Aircon_Compressor_Fan_fault", checkDcAirConCompressorFanFault(model));
        packetModelMap.put("Mains_On_Battery_Discharge", getMainsOnBatteryDischarge(model));

        String socBattery1 = ((int) model.getSocBattery1()) <= 28 ? "1" : "0";
        packetModelMap.put("Soc_Battery_1", socBattery1);

    }

    private String checkBatteryFault(PacketModel model) {
        String returnValue = "0";
        try {
            if (model.getLiBattStatus1().equals("4") || model.getLiBattStatus2().equals("4") || model.getLiBattStatus3().equals("4")
                    || model.getLiBattStatus4().equals("4") || model.getLiBattStatus5().equals("4") || model.getLiBattStatus6().equals("4")
                    || model.getLiBattStatus7().equals("4") || model.getLiBattStatus8().equals("4") || model.getLiBattStatus9().equals("4")
                    || model.getLiBattStatus10().equals("4") || model.getLiBattStatus11().equals("4") || model.getLiBattStatus12().equals("4")
                    || model.getLiBattStatus13().equals("4") || model.getLiBattStatus14().equals("4") || model.getLiBattStatus15().equals("4")
                    || model.getLiBattStatus16().equals("4")) {
                returnValue = "1";
            }
        } catch (Exception ex) {
            System.out.println("Exception at : " + model.getSiteID());
            ex.printStackTrace();
            returnValue = "0";
        }
        return returnValue;
    }

    private String checkPvModuleFail(PacketModel model) {
        String returnValue = "0";
        try {
            if (model.isPv1ModuleStatus() == 1 || model.isPv2ModuleStatus() == 1 || model.isPv3ModuleStatus() == 1 || model.isPv4ModuleStatus() == 1
                    || model.isPv5ModuleStatus() == 1 || model.isPv6ModuleStatus() == 1 || model.isPv7ModuleStatus() == 1 || model.isPv8ModuleStatus() == 1
                    || model.isPv9ModuleStatus() == 1 || model.isPv10ModuleStatus() == 1 || model.isPv11ModuleStatus() == 1 || model.isPv12ModuleStatus() == 1) {
                returnValue = "1";
            }
        } catch (Exception ex) {
            System.out.println("Exception at : " + model.getSiteID());
            ex.printStackTrace();
            returnValue = "0";
        }
        return returnValue;
    }

    private String checkLiCommFault(PacketModel model) {
        String returnValue = "0";
        try {
            if (model.getLiCommFault1().equals("1") || model.getLiCommFault2().equals("1") || model.getLiCommFault3().equals("1")
                    || model.getLiCommFault4().equals("1") || model.getLiCommFault5().equals("1") || model.getLiCommFault6().equals("1")
                    || model.getLiCommFault7().equals("1") || model.getLiCommFault8().equals("1") || model.getLiCommFault9().equals("1")
                    || model.getLiCommFault10().equals("1") || model.getLiCommFault11().equals("1") || model.getLiCommFault12().equals("1")
                    || model.getLiCommFault13().equals("1") || model.getLiCommFault14().equals("1") || model.getLiCommFault15().equals("1")
                    || model.getLiCommFault16().equals("1")) {
                returnValue = "1";
            }
        } catch (Exception ex) {
            System.out.println("Exception at : " + model.getSiteID());
            ex.printStackTrace();
            returnValue = "0";
        }

        return returnValue;
    }

    private String checkDcAirConCompressorFanFault(PacketModel model) {
        String returnValue = "0";
        try {
            if (model.isDcAircon1CompressorFaultFanfault() == 1 || model.isDcAircon2CompressorFaultFanfault() == 1) {
                returnValue = "1";
            }
        } catch (Exception ex) {
            System.out.println("Exception at : " + model.getSiteID());
            ex.printStackTrace();
            returnValue = "0";
        }
        return returnValue;
    }

    public String getMainsOnBatteryDischarge(PacketModel model) {
        String returnValue = "0";
        try {
            if (model.isLoadOnMains() == 1 && model.isBatteryOnDischarge() == 1) {
                returnValue = "1";
            }
        } catch (Exception ex) {
            System.out.println("Exception at : " + model.getSiteID());
            ex.printStackTrace();
            returnValue = "0";
        }
        return returnValue;
    }

    private void sendEscalationAlertEmail(String[] toEmail, String subject, String message) {
        try {
            System.out.println("SENDING EMAIL TO: " + toEmail);
            MimeMessage mimeMessage = alertJavaMailSender.createMimeMessage();
            MimeMessageHelper messageHelper = new MimeMessageHelper(mimeMessage, true, StandardCharsets.UTF_8.toString());
            messageHelper.setSubject(subject);
            messageHelper.setText(message, true);
            messageHelper.setFrom(alertFromAddress);
//            FileSystemResource file = new FileSystemResource(new File(Path));
//            messageHelper.addAttachment(attachmentName, file);
            for (String s : toEmail) {
                messageHelper.addTo(s);
            }
            alertJavaMailSender.send(mimeMessage);

            System.out.println("EMAIL SENT");
        } catch (MessagingException ex) {
            ex.printStackTrace();
            throw new InternalServerException("Error Sending mail");
        } catch (Exception ex) {
            throw new InternalServerException(ex.getMessage());
        }
    }

    // Refresh contract map from DB
    public void refreshContractMap() {
        List<ContractDTO> allContracts = rmsCoreClientService.getAllContracts();
        contractMap = allContracts.stream()
                .collect(Collectors.toMap(ContractDTO::getId, contract -> contract));
    }

    // Get contract by ID (refresh if not found)
    public ContractDTO getContractById(Integer contractId) {
        // Check if contract exists in map
        ContractDTO contract = contractMap.get(contractId);

        if (contract == null) {  // If not found, refresh map
            refreshContractMap();
            contract = contractMap.get(contractId); // Try again
        }

        return contract;  // May return null if contract doesn't exist in DB
    }
}
