package com.ieng.rms.util;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.ieng.rms.entity.AlarmDetailConfig;
import com.ieng.rms.entity.AlarmDetails;
import com.ieng.rms.entity.EscalationEmailEntity;
import com.ieng.rms.entity.SiteWiseAlarmMaskingConfig;
import com.ieng.rms.models.PacketModel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.ieng.rms.util.Constants.*;

@Component
@Slf4j
public class RedisUtil {

    @Autowired
    RedisTemplate<Object, Object> redisTemplate;

    private final RedisTemplate<String, String> stringRedisTemplate;

    private static HashOperations<Object, String, Object> hashOperations;
    private final ObjectMapper objectMapper;

    public RedisUtil(RedisTemplate<String, String> stringRedisTemplate, RedisTemplate<Object, Object> redisTemplate, ObjectMapper objectMapper) {
        hashOperations = redisTemplate.opsForHash();
        this.stringRedisTemplate = stringRedisTemplate;
        this.objectMapper = objectMapper;
        this.objectMapper.registerModule(new JavaTimeModule());


    }

    public String generateKeyForPacketHistory(String siteOrClientId) {
        return REDIS_PACKET_HISTORY_PREFIX + siteOrClientId;
    }

    public void addToRedisForPacketHistory(String key, String dateTime) {
        if (Boolean.TRUE.equals(stringRedisTemplate.hasKey(key))) {
            stringRedisTemplate.opsForValue().getAndSet(key, dateTime);
        } else {
            // If the key doesn't exist, add the new data
            stringRedisTemplate.opsForValue().set(key, dateTime);
        }
    }

    public void putToRedis(String key, String siteOrClientId, Object data) {
        hashOperations.put(generateRedisKey(key, siteOrClientId), generateRedisKey(key, siteOrClientId), data);
        redisTemplate.expire(generateRedisKey(key, siteOrClientId), 31, TimeUnit.DAYS);
    }

    public String getHistoryPacketByKey(String key) {
        return stringRedisTemplate.opsForValue().get(key);
    }

    public static String getSiteOrClientId(PacketModel model, Boolean multiTenant) {
        return multiTenant ? model.getClientId() : model.getSiteID();
    }

    public static String generateRedisKey(String key, String siteOrClientId) {
        return key + "_" + siteOrClientId.toUpperCase();
    }


    /**
     * Saves alarm masking config map to Redis
     */
    public void saveAlarmMaskingConfigToRedis(String siteId, Map<String, SiteWiseAlarmMaskingConfig> data) {
        try {
            // Convert Hibernate proxies to normal objects
            Map<String, SiteWiseAlarmMaskingConfig> detachedData = data.entrySet().stream()
                    .collect(Collectors.toMap(Map.Entry::getKey, entry -> detachEntity(entry.getValue())));

            String jsonData = objectMapper.writeValueAsString(detachedData);
            hashOperations.put(REDIS_ALARM_MASKING_KEY, siteId, jsonData);
            redisTemplate.expire(REDIS_ALARM_MASKING_KEY, 31, TimeUnit.DAYS);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Detach Hibernate proxy by copying values into a new instance.
     */
    private SiteWiseAlarmMaskingConfig detachEntity(SiteWiseAlarmMaskingConfig config) {
        SiteWiseAlarmMaskingConfig detached = new SiteWiseAlarmMaskingConfig();
        detached.setId(config.getId());
        detached.setAlarmName(config.getAlarmName());
        detached.setAlarmMaskingName(config.getAlarmMaskingName());
        detached.setCriticality(config.getCriticality());
        detached.setContractId(config.getContractId());
        detached.setSiteId(config.getSiteId());
        detached.setTranslation(config.getTranslation());
        detached.setEscalationEnable(config.getEscalationEnable());
        detached.setAlarmEnabled(config.getAlarmEnabled());
        detached.setIsNocAlarm(config.getIsNocAlarm());
        detached.setCreatedAt(config.getCreatedAt());
        detached.setUpdatedAt(config.getUpdatedAt());

        // Deep copy escalations to detach from Hibernate session
        List<EscalationEmailEntity> detachedEscalations = config.getEscalations().stream().map(escalation -> {
            EscalationEmailEntity e = new EscalationEmailEntity();
            e.setId(escalation.getId());
            e.setEmail(escalation.getEmail());
            return e;
        }).collect(Collectors.toList());

        detached.setEscalations(detachedEscalations);

        return detached;
    }


    /**
     * Retrieves alarm masking config map from Redis
     */
    public Map<String, SiteWiseAlarmMaskingConfig> getAlarmMaskingConfigFromRedis(String siteId) {
        try {
            String jsonData = (String) hashOperations.get(REDIS_ALARM_MASKING_KEY, siteId);
            return jsonData != null ? objectMapper.readValue(jsonData, new TypeReference<>() {
            }) : null;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public boolean deleteAlarmMaskingConfig(String siteId) {
        try {
            // Delete the field (siteId) from the hash stored under REDIS_ALARM_MASKING_KEY
            Long removedCount = hashOperations.delete(REDIS_ALARM_MASKING_KEY, siteId);
            return removedCount != null && removedCount > 0;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }


    // ✅ Save AlarmDetailConfigMap (String values)
    public void saveAlarmDetailConfigMap(Map<String, String> alarmDetailConfigMap) {
        hashOperations.putAll(REDIS_ALARM_CONFIG_KEY, alarmDetailConfigMap);
        redisTemplate.expire(REDIS_ALARM_CONFIG_KEY, 30, TimeUnit.DAYS);
    }

    // ✅ Save AlarmConfigMap (AlarmDetailConfig objects)
    public void saveAlarmConfigForNocMap(Map<String, AlarmDetailConfig> alarmConfigMap) {
        hashOperations.putAll(REDIS_ALARM_CONFIG_NOC_ALARM_KEY, alarmConfigMap);
        redisTemplate.expire(REDIS_ALARM_CONFIG_NOC_ALARM_KEY, 30, TimeUnit.DAYS);
    }

    // ✅ Get AlarmDetailConfigMap from Redis
    public Map<String, String> getAlarmDetailConfigMap() {
        return hashOperations.entries(REDIS_ALARM_CONFIG_KEY).entrySet().stream()
                .collect(Collectors.toMap(Map.Entry::getKey, e -> (String) e.getValue()));
    }

    // ✅ Get AlarmConfigMap from Redis
    public Map<String, AlarmDetailConfig> getAlarmConfigForNocMap() {
        Map<String, Object> redisData = hashOperations.entries(REDIS_ALARM_CONFIG_NOC_ALARM_KEY);

        return redisData.entrySet().stream()
                .collect(Collectors.toConcurrentMap(
                        Map.Entry::getKey,
                        entry -> objectMapper.convertValue(entry.getValue(), AlarmDetailConfig.class) // Deserialize properly
                ));
    }

    public void saveAlarmDetailsToRedis(Map<String, Map<String, AlarmDetails>> alarmDetailMap) {
        try {
            String jsonValue = objectMapper.writeValueAsString(alarmDetailMap);
            redisTemplate.opsForValue().set(REDIS_ALARM_DETAILS_KEY, jsonValue, 30, TimeUnit.DAYS);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Map<String, Map<String, AlarmDetails>> getAlarmDetailsFromRedis() {
        try {
            Object rawData = redisTemplate.opsForValue().get(REDIS_ALARM_DETAILS_KEY);

            if (rawData != null) {
                return objectMapper.readValue(rawData.toString(), new TypeReference<>() {
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ConcurrentHashMap<>();  // Return empty map if not found
    }

    public List<AlarmDetailConfig> getAlarmDetailConfigFromRedis() {
        try {
            Object rawData = redisTemplate.opsForValue().get(REDIS_ALL_ALARM_DETAIL_CONFIG_KEY);

            if (rawData != null) {
                return objectMapper.readValue(rawData.toString(), new TypeReference<List<AlarmDetailConfig>>() {
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>(); // Return empty list if not found
    }

    public void saveAlarmDetailConfigToRedis(List<AlarmDetailConfig> alarmDetailConfigList) {
        try {
            String jsonValue = objectMapper.writeValueAsString(alarmDetailConfigList);
            redisTemplate.opsForValue().set(REDIS_ALL_ALARM_DETAIL_CONFIG_KEY, jsonValue, 30, TimeUnit.DAYS);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void removeAlarmDetailFromRedis(String siteId, String alarmSource) {
        try {
            String jsonData = (String) redisTemplate.opsForValue().get(REDIS_ALARM_DETAILS_KEY);
            if (jsonData == null || siteId == null || alarmSource == null) return;

            Map<String, Map<String, AlarmDetails>> alarmDetailMap =
                    objectMapper.readValue(jsonData, new TypeReference<>() {
                    });

            // Ensure siteId exists
            if (alarmDetailMap.containsKey(siteId)) {
                Map<String, AlarmDetails> siteAlarmMap = alarmDetailMap.get(siteId);
                if (siteAlarmMap == null || siteAlarmMap.isEmpty()) return;

                // Remove the specific alarm
                String formattedAlarmKey = Util.addUnderscoreAgainstSpace(alarmSource);
                siteAlarmMap.remove(formattedAlarmKey);

                // Remove site entry if no alarms remain
                if (siteAlarmMap.isEmpty()) {
                    alarmDetailMap.remove(siteId);
                }

                // Save back to Redis only if changes were made
                redisTemplate.opsForValue().set(
                        REDIS_ALARM_DETAILS_KEY,
                        objectMapper.writeValueAsString(alarmDetailMap),
                        30, TimeUnit.DAYS
                );
            }
        } catch (Exception e) {
            log.error("Failed to remove alarm from Redis for site: {} and alarm: {}", siteId, alarmSource, e);
        }
    }

    public void deleteKey(String key) {
        redisTemplate.delete(key);
    }

}


