package com.ieng.rms.websocket;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.ieng.rms.models.MonitorAlarmModel;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
public class AlarmMonitorWebsocketHandler extends TextWebSocketHandler {
    private static final Logger logger = LogManager.getLogger(AlarmMonitorWebsocketHandler.class);

    private final ExecutorService executorService = Executors.newFixedThreadPool(10);  // Use a thread pool of 10 threads

    // Map to store sessions grouped by contractId
    private final Map<String, List<WebSocketSession>> sessionsByContractId = new ConcurrentHashMap<>();

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        logger.info("New WebSocket connection established: " + session.getUri());

        // Extract contractId from the session URI
        String contractId = extractContractId(session);
        if (contractId != null) {
            sessionsByContractId
                    .computeIfAbsent(contractId, k -> new CopyOnWriteArrayList<>())
                    .add(session);
            logger.info("Session added for contractId: " + contractId);
        } else {
            logger.warn("Failed to extract contractId from URI: " + session.getUri());
        }
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) {
        sessionsByContractId.values().forEach(sessions -> sessions.remove(session));
        logger.info("WebSocket connection closed.");
    }

    public void sendPacketToContract(MonitorAlarmModel monitorModel) {
        String contractId = monitorModel.getContractParent().toString();
        if (contractId.isEmpty()) {
            logger.warn("Contract ID is null in AlarmModel. Cannot send packet.");
            return;
        }

        String jsonData;
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            // Register JavaTimeModule for Java 8 date/time support
            objectMapper.registerModule(new JavaTimeModule());
            // Optionally, disable timestamps and use ISO-8601 format
            objectMapper.disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);

            jsonData = objectMapper.writeValueAsString(monitorModel);
        } catch (Exception e) {
            logger.error("Failed to serialize AlarmModel to JSON", e);
            return;
        }

        // Retrieve sessions associated with the contractId
        List<WebSocketSession> targetSessions = sessionsByContractId.get(contractId);
        if (targetSessions != null && !targetSessions.isEmpty()) {
            for (WebSocketSession session : targetSessions) {
                executorService.execute(() -> sendMessageToSession(session, jsonData, contractId));

            }
        } else {
            logger.warn("No active WebSocket sessions found for contractId: {}", contractId);
        }
    }

    private void sendMessageToSession(WebSocketSession session, String jsonData, String contractId) {
        if (session.isOpen()) {
            try {
                session.sendMessage(new TextMessage(jsonData));
                logger.info("Packet sent to contractId: {}", contractId);
            } catch (Exception e) {
                logger.error("Error sending message to session for contractId: {}", contractId, e);
            }
        } else {
            logger.warn("WebSocket session is not open for contractId: {}", contractId);
        }
    }

    private String extractContractId(WebSocketSession session) {
        String uri = session.getUri() != null ? session.getUri().toString() : null;
        if (uri != null && uri.contains("/alarm-stream-")) {
            return uri.substring(uri.lastIndexOf("-") + 1);
        }
        return null;
    }
}
